/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.KeyStroke;
import javafx.ui.Icon;
import javafx.ui.Insets;
import javafx.ui.HorizontalAlignment;
import javafx.ui.VerticalAlignment;

public class CheckBox extends Widget {
    private attribute jcheckbox: <<javax.swing.JCheckBox>>;
    public attribute text: String;
    public attribute mnemonic: KeyStroke?;
    public attribute icon: Icon?;
    public attribute selectedIcon: Icon?;
    public attribute pressedIcon: Icon?;
    public attribute rolloverIcon: Icon?;
    public attribute rolloverSelectedIcon: Icon?;
    public attribute disabledIcon: Icon?;
    public attribute disabledSelectedIcon: Icon?;
    public attribute iconTextGap: Number?;
    public attribute contentAreaFilled: Boolean?;
    public attribute focusPainted:Boolean?;
    public attribute borderPainted:Boolean?;
    public attribute borderPaintedFlat: Boolean?;
    public attribute action: function();
    public attribute selected: Boolean;
    public attribute margin:Insets?;
    public attribute horizontalTextPosition:HorizontalAlignment?;
    public attribute verticalTextPosition:VerticalAlignment?;
    public attribute onChange: function(newValue:Boolean)?;
        
}

trigger on (CheckBox.horizontalTextPosition = a) {
    jcheckbox.setHorizontalTextPosition(a.id);
}

trigger on (CheckBox.verticalTextPosition = a) {
    jcheckbox.setVerticalTextPosition(a.id);
}


trigger on (CheckBox.selected = value) {
    if (value <> null) {
        jcheckbox.selected = value;
    }
}

operation CheckBox.createComponent() {
    return jcheckbox;
}

trigger on (new CheckBox) {
    jcheckbox = new <<javax.swing.JCheckBox>>();
	jcheckbox.setOpaque(false);
    var self = this;
    jcheckbox.addActionListener(new <<java.awt.event.ActionListener>>() {
                                    operation actionPerformed(e) {
                                        var selected = self.jcheckbox.isSelected();
                                        self.selected = selected;
                                        (self.onChange)(selected);
                                        (self.action)();
                                    }
                                });
     
}

trigger on (CheckBox.focusPainted = value) {
    jcheckbox.setFocusPainted(value);
}

trigger on (CheckBox.borderPainted = value) {
    jcheckbox.setBorderPainted(value);
}

trigger on (CheckBox.margin = value) {
    jcheckbox.setMargin(value.awtinsets);
}

trigger on (CheckBox.borderPaintedFlat = value) {
    jcheckbox.setBorderPaintedFlat(value);
}

trigger on (CheckBox.contentAreaFilled = value) {
    jcheckbox.setContentAreaFilled(value);
}

trigger on (CheckBox.icon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setIcon(icon);
}

trigger on (CheckBox.rolloverIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setRolloverIcon(icon);
}

trigger on (CheckBox.rolloverSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setRolloverSelectedIcon(icon);
}

trigger on (CheckBox.selectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setSelectedIcon(icon);
}

trigger on (CheckBox.pressedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setPressedIcon(icon);
}

trigger on (CheckBox.disabledIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setDisabledIcon(icon);
}

trigger on (CheckBox.disabledSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    jcheckbox.setDisabledSelectedIcon(icon);
}

trigger on (CheckBox.iconTextGap = value) {
    jcheckbox.setIconTextGap(value);
}

trigger on (CheckBox.text = value) {
    jcheckbox.setText(value);
}
