/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.InternalFrameDragMode;
import javafx.ui.InternalFrame;

public class DesktopPane extends Widget {
    private attribute jdesk: <<javax.swing.JDesktopPane>>;

    public attribute dragMode: InternalFrameDragMode;
    public attribute frames: InternalFrame*;
    public attribute cascaded: Boolean;
    public attribute tiled: Boolean;
}

attribute DesktopPane.dragMode = LIVE:InternalFrameDragMode;

trigger on DesktopPane.dragMode = value {
    jdesk.setDragMode(value.id);
}

operation DesktopPane.createComponent() {
    jdesk = new <<javax.swing.JDesktopPane>>();
    jdesk.setOpaque(false);
    jdesk.setDragMode(dragMode.id);
    for (i in frames) {
        var count = jdesk.getComponentCountInLayer(i.layer);
        var b = new <<java.awt.Rectangle>>(i.x, i.y, i.width, i.height);
        jdesk.add(i.getComponent(), i.layer.intValue(), 0);
        i.setBounds(b);
        i.getComponent().setVisible(true);
    }
    return jdesk;
}

trigger on (insert frame into DesktopPane.frames) {
    frame.desk = this;
    var i = frame;
    var b = new <<java.awt.Rectangle>>(i.x, i.y, i.width, i.height);
    jdesk.add(frame.getComponent(), frame.layer.intValue(), 0);
    i.setBounds(b);
    frame.getComponent().setVisible(true);
}


trigger on (delete frame from DesktopPane.frames) {
    jdesk.remove(frame.getComponent());
}


trigger on (DesktopPane.cascaded = value) {
    if (value <> null) {
        if (value) {
            var frames = jdesk.getAllFrames();
            var x = 0.0;
            var y = 0.0;
            var w = 0.0;
            var h = jdesk.getHeight();
            var d = (h / sizeof frames).intValue();
            if (d < 15) {
                d = 15;
            }
            for (f in reverse frames) {
                try {
                    f.setVisible(true);
                    f.setIcon(false);
                } catch (ignore) {
                }
                try {
                    f.setMaximum(false);
                } catch (ignore) {
                }
                var dim = f.getSize();
                w = dim.width;
                h = dim.height;
                jdesk.getDesktopManager().setBoundsForFrame(f, x, y, w, h);
                x += d;
                y += d;
            }
            cascaded = null;
        }
    }
}

trigger on (DesktopPane.tiled = value) {
    if (value <> null) {
        if (value) {
            var frames = jdesk.allFrames;
            var rows = <<java.lang.Math>>.sqrt(sizeof frames).intValue();
            var cols = rows;
            if (rows * cols < sizeof frames) {
                cols += 1;
                if (rows * cols < sizeof frames) {
                    rows += 1;
                }
            }
            var size = jdesk.getSize();
            var w = (size.width / cols).intValue();
            var h = (size.height/ rows).intValue();
            var x = 0;
            var y = 0;
            for (i in [0..<rows]) {
                for (j in [0..<cols]) {
                    var cell = (i * cols) + j;
                    if (cell >= sizeof frames) {
                        break;
                    }
                    var f = frames[cell];
                    try {
                        f.setVisible(true);
                        f.setIcon(false);
                    } catch (ignore) {
                    }
                    try {
                        f.setMaximum(false);
                    } catch (ignore) {
                    }
                    jdesk.getDesktopManager().setBoundsForFrame(f, x, y, w, h);
                    x += w;
                }
                y += h;
                x = 0;
            }
            tiled = null;
        }
    }
}
