/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.AbstractFrame;
import javafx.ui.UIElement;
import javafx.ui.Button;
import javafx.ui.Widget;
import javafx.ui.Border;
import javafx.ui.EmptyBorder;

public class Dialog extends AbstractFrame {
    private attribute jdialog:<<javax.swing.JDialog>>;
    private attribute p:<<javax.swing.JPanel>>;
    private attribute buttonpanel: <<javax.swing.JPanel>>;

    public attribute modal:Boolean?;
    public attribute owner: UIElement;
    public attribute title: String;
    public attribute content:Widget;
    public attribute buttons: Button*;
    public attribute height:Number?;
    public attribute width: Number?;
    public attribute border: Border;
    public attribute visible: Boolean;
    public operation show();
    public operation showDialog(w:UIElement);
    public operation hide();
    public attribute onClose: function();
}


trigger on (new Dialog) {
    p = new <<javax.swing.JPanel>>();
    p.setLayout(new <<java.awt.BorderLayout>>());
    border = EmptyBorder {top: 10 left: 10 right: 10 bottom: 10};
}

trigger on (Dialog.border = b) {
    p.setBorder(b.getBorder());
}

trigger on (insert button into Dialog.buttons) {
    if (buttonpanel == null) {
        buttonpanel = new <<javax.swing.JPanel>>();
        buttonpanel.setLayout(new <<javax.swing.BoxLayout>>(buttonpanel, <<javax.swing.BoxLayout>>.X_AXIS));
        buttonpanel.setBorder(<<javax.swing.BorderFactory>>.createEmptyBorder(10, 5, 0, 0));
        buttonpanel.add(<<javax.swing.Box>>.createHorizontalGlue());
        p.add(buttonpanel, <<java.awt.BorderLayout>>.SOUTH);
    }
    buttonpanel.add(<<javax.swing.Box>>.createRigidArea(new <<java.awt.Dimension>>(5,0)));
    buttonpanel.add(button.getComponent());
    var dim = new <<java.awt.Dimension>>(80,0);
    for (i in buttons) {
        var d = i.getComponent().getPreferredSize();
        if (dim.height < d.height) {
            dim.height = d.height;
        }
        if (dim.width < d.width) {
            dim.width = d.height;
        }
    }
    for (i in buttons) {
        i.getComponent().setPreferredSize(dim);
        i.getComponent().setMinimumSize(dim);
        i.getComponent().setMaximumSize(dim);
    }
}

trigger on (Dialog.content = w) {
    p.add(w.getComponent(), <<java.awt.BorderLayout>>.CENTER);
}

trigger on (Dialog.visible = value) {
     if (value) {
        do later {
             this.show();
        }
     } else {
        this.hide();
     }
}


operation Dialog.showDialog(w:UIElement) {
    owner = w;  
    this.show();
}

operation Dialog.show() {
    var win = owner.getWindow();
    if (win instanceof <<java.awt.Dialog>>) {
        jdialog = new <<javax.swing.JDialog>>((<<java.awt.Dialog>>)win);
    } else {
        jdialog = new <<javax.swing.JDialog>>((<<java.awt.Frame>>)win);
    }
    jdialog.setContentPane(p);
    var self = this;
    win.addWindowListener(new <<java.awt.event.WindowListener>>() {
                              operation windowClosing(e) {
                                  var root = self.jdialog.getRootPane();
                                  var but = (<<javax.swing.JButton>>)root.getClientProperty("net.java.javafx.ui.defaultClose");
                                  if (but <> null) {
                                        but.doClick(0);
                                  }
                                  (self.onClose)();
                                  self.jdialog.dispose();
                                  self.jdialog = null;
                              }
                          });
    if (modal <> null) {
        jdialog.setModal(modal);
    }
    jdialog.setTitle(title);
    jdialog.pack();
    var dim = jdialog.getSize();
    if (height <> null) {
        dim.height = height;
    }
    if (width <> null) {
        dim.width = width;
    }
    jdialog.setSize(dim);
    if (win <> null) {
        jdialog.setLocationRelativeTo(win);
    }
    jdialog.requestFocus();
    jdialog.show();
}

operation Dialog.hide() {
    jdialog.dispose();
    jdialog = null;
}
