/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.FileFilter;
import java.io.File;

public class FileChooser extends Widget {
    private attribute propertyChangeListener: <<java.beans.PropertyChangeListener>>;
    private attribute filechooser: <<javax.swing.JFileChooser>>;
    public attribute cwd: File?;
    public attribute fileFilters: FileFilter*;
    public attribute title: String;
    public attribute files: Boolean?;
    public attribute directories:Boolean?;
    public attribute action: function(selectedFile:File);
    public attribute selectedFile: File?;
    public attribute accessory: Widget?;
    public attribute controlButtonsAreShown: Boolean;
    public operation showOpenDialog(w:UIElement);
    public operation showSaveDialog(w:UIElement);
    protected operation createFileChooser():<<javax.swing.JFileChooser>>;
    protected operation configure();
}

operation FileChooser.configure() {
    filechooser.setDialogTitle(title);
    filechooser.resetChoosableFileFilters();
    filechooser.setAccessory(accessory.getComponent());
    filechooser.setControlButtonsAreShown(controlButtonsAreShown);
    var f = if files <> null then files else true;
    var d = if directories <> null then directories else false;
    var mode = if (f and d) then filechooser.FILES_AND_DIRECTORIES else if d then filechooser.DIRECTORIES_ONLY else filechooser.FILES_ONLY;
    filechooser.setFileSelectionMode(mode);
    var self = this;
    for(i in reverse fileFilters) {
        context:<<net.java.javafx.ui.UIContext>>.addChoosableFileFilter(filechooser, new <<net.java.javafx.ui.FileFilter>>() {
                                               operation equals(obj) {
                                                  return obj == this;
                                               }
                                               operation accept(f:File):Boolean {
                                                   
                                                   return (i.filter)(f);
                                               }
                                               operation getDescription():String {
                                                   return i.description;
                                               }
                                           });
    }
    if (cwd <> null) {
        filechooser.setCurrentDirectory(cwd.getCanonicalFile());
    }
}

trigger on (new FileChooser) {
    controlButtonsAreShown = true;
    filechooser = this.createFileChooser();
    var self = this;
    propertyChangeListener = new <<java.beans.PropertyChangeListener>>() {
        operation propertyChange(e:<<java.beans.PropertyChangeEvent>>) {
            var prop = e.propertyName;
            if (prop == <<javax.swing.JFileChooser>>.SELECTED_FILE_CHANGED_PROPERTY) {
                self.selectedFile = (File)e.getNewValue();
            }
        }
    };
}

operation FileChooser.getComponent() {
    this.configure();
    return filechooser;
}

operation FileChooser.createFileChooser() {
    return context:<<net.java.javafx.ui.UIContext>>.createFileChooser();
}

operation FileChooser.showOpenDialog(w:UIElement) {
    this.configure();
    filechooser.addPropertyChangeListener(propertyChangeListener);
    var rc;
    try {
        rc = filechooser.showOpenDialog(w.getWindow());
    } finally {
        filechooser.removePropertyChangeListener(propertyChangeListener);
    }
    if (rc == filechooser.APPROVE_OPTION) {
        (this.action)(filechooser.getSelectedFile());
        cwd = filechooser.getCurrentDirectory();
    }
}

operation FileChooser.showSaveDialog(w:UIElement) {
    this.configure();
    if (filechooser.showSaveDialog(w.getWindow()) == filechooser.APPROVE_OPTION) {
        (this.action)(filechooser.getSelectedFile());
        cwd = filechooser.getCurrentDirectory();
    }
}
