/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Alignment;

public class FlowPanel extends Widget {
    private attribute panel: <<net.java.javafx.ui.ScrollablePanel>>;
    private attribute layout: <<java.awt.FlowLayout>>;
    protected operation createLayout();
    public attribute alignment:Alignment;
    public attribute vgap: Number;
    public attribute hgap: Number;
    public attribute content: Widget*;
}

trigger on (new FlowPanel) {
    alignment = CENTER:Alignment;
    vgap = 5;
    hgap = 5;
}

operation FlowPanel.createLayout() {
    for (i in [0..<panel.getComponentCount()]) {
        panel.remove(0);
    }
    var align = 
	if (alignment == LEADING:Alignment)
	then <<java.awt.FlowLayout>>.LEADING
        else if (alignment == TRAILING:Alignment)
	then <<java.awt.FlowLayout>>.TRAILING
        else if (alignment == CENTER:Alignment)
        then <<java.awt.FlowLayout>>.CENTER
        else <<java.awt.FlowLayout>>.LEADING; // default
    layout = new <<net.java.javafx.ui.FlowLayout>>(align, hgap, vgap);
    panel.setLayout(layout);
    for (i in content) {
        panel.add(i.getComponent());
    }
    return panel;
}

operation FlowPanel.createComponent() {
    panel = new <<net.java.javafx.ui.ScrollablePanel>>();
    panel.setScrollableTracksViewportWidth(true);
    panel.setScrollableTracksViewportHeight(false);
    panel.setOpaque(false);
    this.createLayout();
    return panel;
}

trigger on (FlowPanel.alignment = value) {
    if (panel <> null) {
        this.createLayout();
    }
}

trigger on (insert w into FlowPanel.content) {
    if (panel <> null) {
        panel.add(w.getComponent(), indexof w);
        panel.revalidate();
        panel.repaint();
    }
}

trigger on (delete w from FlowPanel.content) {
    if (panel <> null) {
        panel.remove(indexof w);
        panel.revalidate();
        panel.repaint();
    }
}

trigger on (FlowPanel.content[oldValue] = w) {
    if (panel <> null) {
        panel.remove(indexof w);
        panel.add(w.getComponent(), indexof w);
        panel.revalidate();
        panel.repaint();
    }
}

trigger on (FlowPanel.vgap = value) {
    layout.setVgap(value);
}

trigger on (FlowPanel.hgap = value) {
    layout.setHgap(value);
}
