/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.GroupElement;
import javafx.ui.Alignment;
import javafx.ui.Row;
import javafx.ui.Column;
import javafx.ui.Widget;
import javafx.ui.Gap;

/**
 * Interface for elements that perform group layout.
 */

public class GroupLayout extends GroupElement {
    /** Vertical alignment of the contained elements. Defaults to BASELINE.*/
    public attribute valign: Alignment;
    /** Horizontal alignment of the contained elements. Defaults to LEADING.*/
    public attribute halign: Alignment;
    /** The row definitions for this layout. */
    public attribute rows: Row*;
    /** The column definitions for this layout. */
    public attribute columns: Column*;
    /** The contents of this layout. */
    public attribute content: GroupElement*;

    private operation addRows(host:<<java.awt.Container>>,layout:<<org.jdesktop.layout.GroupLayout>>, vgroup:<<org.jdesktop.layout.GroupLayout$SequentialGroup>>);
    private operation addColumns(host:<<java.awt.Container>>, layout:<<org.jdesktop.layout.GroupLayout>>, hgroup:<<org.jdesktop.layout.GroupLayout$SequentialGroup>>);
    private operation addComponents(host:<<java.awt.Container>>);
}

attribute GroupLayout.valign = BASELINE:Alignment;
attribute GroupLayout.halign = LEADING:Alignment;

operation GroupLayout.addComponents(host:<<java.awt.Container>>) {
    for (e in content where e instanceof GroupLayout) {
        ((GroupLayout)e).addComponents(host);
    }
    for (e in content where e instanceof Widget) {
        host.add(((Widget)e).getComponent());
    }
}

operation GroupLayout.addRows(host:<<java.awt.Container>>,layout:<<org.jdesktop.layout.GroupLayout>>, vgroup:<<org.jdesktop.layout.GroupLayout$SequentialGroup>>) {
    if (rows == null and content.row == null) {
        rows = Row {};
        for (i in content) {
            i.row = rows[0];
        }
    }
    for (i in rows) {
        var pgroup = layout.createParallelGroup(i.alignment.id, i.resizable);
        vgroup.add(pgroup);
        var comps:<<java.awt.Component>> = [];
        for (j in content where j.row == i) {
            if (j instanceof Widget) {
                var w = (Widget)j;
                if (w.sizeToFitRow) {
                    insert w.getComponent() into comps;
                }
                var c = w.getComponent();
                if (c == null) {
                    throw "Component was null for Widget {w}";
                }
                if (j.vertical <> null) {
                    var spring = j.vertical;
                    pgroup.add(c, spring.min, spring.pref, spring.max);
                } else if (i.resizable) {
                    pgroup.add(c,  
                               0,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               MAX_INT:Number);
                } else {
                    pgroup.add(c, 
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE);
                }
            } else if (j instanceof Gap) {
                var spring = j.vertical;
                pgroup.add(spring.min, spring.pref, spring.max);
            } else if (j instanceof GroupLayout) {
                var sg = layout.createSequentialGroup(); 
                var g = (GroupLayout)j;
                g.addRows(host, layout, sg);
                pgroup.add(i.alignment.id, sg);
            }
        }
        if (sizeof comps > 0) {
            layout.linkSize(comps, layout.VERTICAL);
        }
    }
}

operation GroupLayout.addColumns(host:<<java.awt.Container>>, layout:<<org.jdesktop.layout.GroupLayout>>, hgroup:<<org.jdesktop.layout.GroupLayout$SequentialGroup>>) {
    if (columns == null and content.column == null) {
       columns = Column {};
       for (i in content) {
           i.column = columns[0];
       }
    }
    for (i in columns) {
        var pgroup = layout.createParallelGroup(i.alignment.id, i.resizable);
        hgroup.add(pgroup);
        var comps:<<java.awt.Component>> = [];
        for (j in content where j.column == i) {
            if (j instanceof Widget) {
                var w = (Widget)j;
                var c = w.getComponent();
                if (c == null) {
                    throw "Component was null for Widget {w}";
                }
                if (w.sizeToFitColumn) {
                    insert c into comps;
                }
                if (j.horizontal <> null) {
                    var spring = j.horizontal;
                    pgroup.add(c, spring.min, spring.pref, spring.max);
                } else if (i.resizable) {
                    pgroup.add(c,  
                               0,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               MAX_INT:Number);
                } else {
                    pgroup.add(c, 
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE,
                               <<org.jdesktop.layout.GroupLayout>>.PREFERRED_SIZE);
                }
            } else if (j instanceof Gap) {
                var spring = j.horizontal;
                pgroup.add(spring.min, spring.pref, spring.max);
            } else if (j instanceof GroupLayout) {
                var g = (GroupLayout)j;
                var sg = layout.createSequentialGroup(); 
                g.addColumns(host, layout, sg);
                pgroup.add(i.alignment.id, sg);
            }
        }
        if (sizeof comps > 0) {
            layout.linkSize(comps, layout.HORIZONTAL);
        }
    }
}
