/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Icon;
import java.awt.Dimension;
import net.java.javafx.ui.ImageDownloadNotifier;
import net.java.javafx.ui.ImageDownloadObserver;
/**
 * An implementation of the Icon interface that paints Icons
 * from Images. Images are created from a URL
 */

public class Image extends Icon {
    private attribute notifier: <<net.java.javafx.ui.ImageLoadingNotifier>>;
    protected attribute image:<<java.awt.Image>>?;
    attribute baseURL:String?;
    private operation getImage0();
    public attribute preloadIfLocal: Boolean?;
    public attribute url: String?;
    public attribute size: Dimension?;
    public attribute onLoad: function();
    public attribute stretch: Stretch?;
    public attribute totalDownloadSize: Number;
    public attribute totalDownloaded: Number;
    public attribute stretchDirection: StretchDirection?;
    public attribute downloadProgress: function(totalRead:Number, ofTotal:Number);
    private operation installListener();
    public operation getURL():<<java.net.URL>>;
}


attribute Image.preloadIfLocal = true;

operation Image.getURL() {
    return context:<<net.java.javafx.ui.UIContext>>.getImageURL(url);
}

trigger on (Image.url = value) {
    image = null;
    this.getImage0();
}

trigger on (Image.onLoad = value) {
//    this.installListener();
}

operation Image.installListener() {
    if (image <> null and onLoad <> null) {
        var self = this;
        notifier = new <<net.java.javafx.ui.ImageLoadingNotifier>>(image,
                                                new <<net.java.javafx.ui.ImageLoadingNotifier$ImageLoadingListener>>() {
                                                    operation imageLoadingDone(img) {

                                                        (self.onLoad)();
                                                        self.notifier = null;
                                                    }
                                                    operation imageLoadingFailed(img) {

                                                        (self.onLoad)();
                                                        self.notifier = null;
                                                    }
                                                });
    }
}


trigger on (Image.size = value) {
}

function Image.getIcon() {
    var im = this.getImage();
    return if im == null then null else  new <<javax.swing.ImageIcon>>(im);
}

function Image.getImage() {
    return if image == null then getImage0() else image;
}

operation Image.getImage0() {
    if (image == null and url <> null) {
        var imageUrl = url;
        if (baseURL <> null) {
            var base = new <<java.net.URL>>(baseURL);
            imageUrl = new <<java.net.URL>>(base, url).toString(); 
        }
        image = if (preloadIfLocal and (url.startsWith("data:") or url.startsWith("file:") or url.startsWith("jar:"))) 
            then context:<<net.java.javafx.ui.UIContext>>.getLoadedImage(imageUrl)
            else context:<<net.java.javafx.ui.UIContext>>.getImage(imageUrl);
        if (image.getSource() instanceof ImageDownloadNotifier) {
            var notifier = (ImageDownloadNotifier)image.getSource();
            var self = this;
            totalDownloadSize = notifier.totalSize;
            totalDownloaded = notifier.totalRead;
            notifier.addImageDownloadObserver(new ImageDownloadObserver() {
                    operation progress(totalRead:Number, ofTotal:Number) {
                        do later {
                            self.totalDownloadSize = ofTotal;
                            self.totalDownloaded = totalRead;
                            (self.downloadProgress)(totalRead, ofTotal);
                        }
                    }
                });
            if (notifier.totalRead > 0 and notifier.totalRead == notifier.totalSize) {
                (self.downloadProgress)(notifier.totalRead, notifier.totalSize);
            }
        }
        
        if (image <> null and size <> null) {
            new <<javax.swing.ImageIcon>>(image);
            var w = (Number)image.getWidth(null);
            var h = (Number)image.getHeight(null);
            if (false) {
                var t = new <<java.awt.geom.AffineTransform>>();
                t.scale(size.width/w, size.height/h);
                var ge = <<java.awt.GraphicsEnvironment>>.getLocalGraphicsEnvironment();
                var gs = ge.getDefaultScreenDevice();
                var gc = gs.getDefaultConfiguration();
                var im = gc.createCompatibleImage(size.width, size.height,
                                                  <<java.awt.Transparency>>.TRANSLUCENT);
                var g = (<<java.awt.Graphics2D>>)im.getGraphics();
                g.setTransform(t);
                g.drawImage(image, 0, 0, null);
                g.dispose();
                image = im;
            } else {
                var thumb = image;
                if (size.width < size.height) {
                    var ratio = w/h;
                    while (true) {
                        w /= 2;
                        if (w < size.width) {
                            w = size.width;
                        }
                        var temp = new <<java.awt.image.BufferedImage>>(w,  (w / ratio).intValue(), <<java.awt.image.BufferedImage>>.TYPE_INT_ARGB);
                        var g2 = temp.createGraphics();
                        g2.setRenderingHint(<<java.awt.RenderingHints>>.KEY_INTERPOLATION,
                                            <<java.awt.RenderingHints>>.VALUE_INTERPOLATION_BILINEAR);
                        g2.drawImage(thumb, 0, 0, temp.getWidth(), temp.getHeight(), null);
                        g2.dispose();
                        thumb = temp;
                        if (w == size.width) {
                            break;
                        }
                    } 
                } else {
                    var ratio = h/w;
                    while (true) {
                        h /= 2;
                        if (h < size.height) {
                            h = size.height;
                        }
                        var temp = new <<java.awt.image.BufferedImage>>((h / ratio).intValue(), h, <<java.awt.image.BufferedImage>>.TYPE_INT_ARGB);
                        var g2 = temp.createGraphics();
                        g2.setRenderingHint(<<java.awt.RenderingHints>>.KEY_INTERPOLATION,
                                            <<java.awt.RenderingHints>>.VALUE_INTERPOLATION_BILINEAR);
                        g2.drawImage(thumb, 0, 0, temp.getWidth(), temp.getHeight(), null);
                        g2.dispose();
                        thumb = temp;
                        if (h == size.height) {
                            break;
                        }
                    } 
                }
                image = thumb;
            }
        }
    }
    return image;
}
