/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javax.swing.JInternalFrame;
import net.java.javafx.ui.XInternalFrame;
import javax.swing.JPanel;
import java.awt.event.ComponentListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.event.InternalFrameListener;
import net.java.javafx.ui.ScrollablePanel;
import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.JViewport;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import net.java.javafx.ui.UIContext;
import javax.swing.ImageIcon;
import javax.swing.JRootPane;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JApplet;
import javax.swing.JWindow;
import javax.swing.JLayeredPane;


class InternalWindow extends UIElement {
    // private:
    operation getRootPane(e:UIElement):JRootPane;
    attribute contentComponent:JComponent;
    attribute dx: Number;
    attribute dy: Number;
    attribute inBounds: Boolean;
    attribute frame: XInternalFrame;
    attribute glass: JPanel;
    attribute frameBounds: <<java.awt.Rectangle>>;
    attribute oldGlass: <<java.awt.Component>>;
    attribute oldGlassVisible: Boolean;
    attribute sp:ScrollablePanel;
    attribute vp:JViewport;

    // public:
    attribute modal:Boolean;
    attribute background: Color?;
    attribute anchor: Anchor?;
    attribute icon: Icon;
    attribute owner: UIElement;
    attribute title: String;
    attribute visible: Boolean;
    attribute content: Widget;
    attribute closable: Boolean;
    attribute resizable: Boolean;
    attribute x: Number;
    attribute y: Number;
    attribute width: Number;
    attribute height: Number;
    attribute onClose: function();
    attribute fade: Boolean;
    attribute slide: Boolean;
    attribute animationDuration: Number;
    attribute animating: Boolean;
    operation onSetContent(value:Widget);
    operation setContent(value:<<javax.swing.JComponent>>);
}

operation InternalWindow.getWindow() {
    return owner.getWindow();
}

attribute InternalWindow.slide = false;
attribute InternalWindow.resizable = true;
attribute InternalWindow.closable = true;
attribute InternalWindow.animationDuration = 200;
attribute InternalWindow.icon = Icon { icon: UIManager.getIcon("InternalFrame.icon") };

trigger on InternalWindow.background = value {
    frame.getContentPane().setBackground(value.getColor());
}

trigger on InternalWindow.anchor = value {
}

trigger on InternalWindow.resizable = value {
    frame.setResizable(value);
}

trigger on InternalWindow.closable = value {
    frame.setClosable(value);
}

trigger on InternalWindow.icon = value {
    if (icon == null) {
        var defaultIcon = UIManager.getIcon("InternalFrame.icon");
        var emptyIcon = new ImageIcon(context:UIContext.getTransparentImage(defaultIcon.getIconWidth(),
                                                                    defaultIcon.getIconHeight()));
        frame.setFrameIcon(emptyIcon);
    } else {
        frame.setFrameIcon(icon.getIcon());
    }
}

trigger on InternalWindow.title = value {
    frame.setTitle(value);
}

trigger on InternalWindow.content = value {
    this.onSetContent(value);
}

operation InternalWindow.onSetContent(value:Widget) {
    setContent(value.getComponent());
}

operation InternalWindow.setContent(value:<<javax.swing.JComponent>>) {
    contentComponent = value;
    var container = sp;
    if (container <> null) {
        if (container.getComponentCount() > 0) {
            container.remove(0);
        }
        if (value <> null) {
            container.add(value);
        }
        frame.validate();
    }
}

trigger on InternalWindow.x = n {
    if (not inBounds) {
        frame.reshape(x, y, width, height);
        frame.repaint();
    }
}

trigger on InternalWindow.y = n {
    if (not inBounds) {
        frame.reshape(x, y, width, height);
        frame.repaint();
    }
}

trigger on InternalWindow.height = n {
    if (not inBounds) {
        frame.reshape(x, y, width, height);
        frame.repaint();
    }
}

trigger on InternalWindow.width = n {
    if (not inBounds) {
        frame.reshape(x, y, width, height);
        frame.repaint();
    }
}

trigger on (InternalWindow.visible = value) {
    do later {
        if (visible <> value) {
                return;
        }
        if (value) {
            if (glass <> null) {
                return;
            }
        } else {
            if (glass == null) {
                return;
            }
        }
        if (value) {
            dx = x;
            dy = y;
            var root = this.getRootPane(owner);
            oldGlass = root.getGlassPane();
            oldGlassVisible = oldGlass.isVisible();
            frame = context:UIContext.createInternalFrame();
            glass = new JPanel();
            glass.setBounds(0, 0, 5000, 5000); // fix me
            glass.setOpaque(false);
            glass.addMouseListener(new MouseListener());
            frame.setClosable(closable);
            frame.setIconifiable(false);
            frame.setMaximizable(false);
            frame.setResizable(resizable);
            frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
            sp = new ScrollablePanel();
            sp.setLayout(new BorderLayout());
            sp.setOpaque(false);
            vp = new JViewport();
            vp.setView(sp);
            vp.setOpaque(false);
            var p = new JPanel();
            p.setOpaque(false);
            if (background <> null) {
                p.setBackground(background.getColor());
            }
            p.setLayout(new BorderLayout());
            p.add(vp, BorderLayout.CENTER);
            frame.setOpaque(false);
            frame.setVisible(false);
            frame.setContentPane(p);
            frame.setTitle(title);
            if (icon == null) {
                var defaultIcon = UIManager.getIcon("InternalFrame.icon");
                var emptyIcon = new ImageIcon(context:UIContext.getTransparentImage(defaultIcon.getIconWidth(),
                                                                            defaultIcon.getIconHeight()));
                frame.setFrameIcon(emptyIcon);
            } else {
               frame.setFrameIcon(icon.getIcon());
            }
            if (contentComponent <> null) {
                sp.add(contentComponent);
            }
            var self = this;
            if (modal) {
                root.getLayeredPane().add(glass, JLayeredPane.MODAL_LAYER, 0);
            }
            root.getLayeredPane().add(frame, JLayeredPane.MODAL_LAYER, 0);
            frame.pack();
            glass.setVisible(modal);
            if (fade) {
                frame.setOpacity(0.1);
            }
            frame.addComponentListener(new ComponentListener() {
                    operation componentMoved(e) {
                        self.inBounds = true;
                        var b = self.frame.getBounds();
                        self.x = b.x;
                        self.y = b.y;
                        self.inBounds = false;
                    }
                    operation componentResized(e) {
                        self.inBounds = true;
                        var b = self.frame.getBounds();
                        self.width = b.width;
                        self.height = b.height;
                        self.inBounds = false;
                    }
                });
            frame.addInternalFrameListener(new InternalFrameListener() {
                    operation internalFrameClosing(e) {
                        if (self.onClose <> null) {
                            (self.onClose)();
                        } else {
                            self.visible = false;
                        }
                    }
                });
            if (height == 0 or width == 0) {
                var b = frame.getBounds();
                inBounds = true;
                if (width == 0) {
                    width = b.width;
                }
                if (height == 0) {
                    height = b.height;
                }
                inBounds = false;
            } 
            if (anchor <> null) {
                inBounds = true;
                var r = root.getBounds();
                if (anchor == CENTER:Anchor) {
                    x = r.width/2 - width/2;
                    y = r.height/2 - height/2;
                } else if (anchor == NORTH:Anchor) {
                    x = r.width/2 - width/2;
                    y =0;
                } else if (anchor == SOUTH:Anchor) {
                    x = r.width/2 - width/2;
                    y = r.height - height;
                } else if (anchor == SOUTHWEST:Anchor) {
                    x = 0;
                    y = r.height - height;
                } else if (anchor == SOUTHEAST:Anchor) {
                    x = r.width-width;
                    y = r.height - height;
                } else if (anchor == NORTHEAST:Anchor) {
                    x = r.width - width;
                    y = 0;
                } else if (anchor == NORTHWEST:Anchor) {
                    x = 0;
                    y = 0;
                }
                x += dx;
                y += dy;
                inBounds = false;
            }
        } 
        if (not slide and not fade) {
            if (not value) {
                var root = this.getRootPane(owner);
                frame.getParent().remove(frame);
                glass.getParent().remove(glass);
                root.validate();
                root.repaint();
                oldGlass = null;
                glass = null;
                frame = null;
                if (anchor <> null) {
                    x = dx;
                    y = dy;
                }
            } else {
                var rect = new <<java.awt.Rectangle>>(x, y, width, height);
                sp.setScrollableTracksViewportHeight(true);
                sp.setScrollableTracksViewportWidth(true);
                sp.setPreferredScrollableViewportSize(null);
                frame.pack();
                frame.setBounds(rect);
                frame.setSelected(true);
            }
            if (not fade) {
                return;
            }
        }
        if (value) {
            frame.setVisible(not fade);
            sp.setScrollableTracksViewportHeight(not slide);
            sp.setScrollableTracksViewportWidth(not slide);
            if (slide) {
                sp.setPreferredScrollableViewportSize(new Dimension(width-10, 
                                                                    height-35));
            } else {
                sp.setPreferredScrollableViewportSize(null);
            }
            if (anchor == null or anchor in [NORTH:Anchor, NORTHEAST:Anchor, NORTHWEST:Anchor, CENTER:Anchor]) {
                var h = height;
                var rect = new <<java.awt.Rectangle>>(x, y, width, height);
                frame.setBounds(x, y, width, if slide then 0 else height);
                for (i in [0..h]) (dur animationDuration easeboth) {
                    if (fade) {
                        if (i > 0) {
                                var opacity = (i/h).floatValue();
                                frame.setOpacity(opacity);
                                if (not frame.isVisible()) {
                                    frame.setVisible(true);
                                }
                        }
                    }
                    if (slide) {
                        frame.setBounds(x, y, width, i);
                    }
                    animating = i < h;
                    if (i == h and slide) {
                        if (not frame.isSelected()) {
                            frame.setSelected(true);
                        }
                        sp.setScrollableTracksViewportHeight(true);
                        sp.setScrollableTracksViewportWidth(true);
                        sp.setPreferredScrollableViewportSize(null);
                    }
                }
            } else {
                var h = height;
                var y1 = y;
                frame.reshape(x, h, width, if slide then 0 else height);
                for (i in [h..0]) (dur animationDuration easeboth) {
                    if (fade) {
                        frame.setOpacity(1-i/h);
                    }
                    if (slide) {
                        frame.setBounds(x, y1+i, width, h-i);
                    }
                    if (not frame.isSelected()) {
                        frame.setSelected(true);
                    }
                    animating = i > 0;
                    if (i == 0) {
                        sp.setScrollableTracksViewportHeight(true);
                        sp.setScrollableTracksViewportWidth(true);
                        sp.setPreferredScrollableViewportSize(null);
                    }

                }
            }
        } else {
            if (oldGlass == null) {
                return;
            }
            sp.setScrollableTracksViewportHeight(false);
            sp.setScrollableTracksViewportWidth(false);
            sp.setPreferredScrollableViewportSize(new Dimension(width-10, height-35));
            if (anchor == null or anchor in [NORTH:Anchor, NORTHEAST:Anchor, NORTHWEST:Anchor, CENTER:Anchor]) {
                var h = frame.getHeight();
                ((JLayeredPane)glass.getParent()).moveToFront(glass);
                for (i in [h..0]) (dur animationDuration easeboth) {
                    if (frame <> null) {
                        if (slide) {
                            frame.setBounds(x, y, width, i);
                        }
                        if (fade) {
                            frame.setOpacity(i/h);
                        }
                        animating = i > 0;
                        if (i == 0) {
                            var root = this.getRootPane(owner);
                            glass.getParent().remove(glass);
                            frame.getParent().remove(frame);
                            oldGlass = null;
                            glass = null;
                            frame = null;
                            height = h;
                            if (anchor <> null) {
                                x = dx;
                                y = dy;
                            }
                        }
                    }
                }
            } else {
                var h = frame.getHeight();
                var y1 = y;
                for (i in [0..h]) (dur animationDuration easeboth) {
                    if (frame <> null) {
                        if (slide) {
                            frame.setBounds(x, y1 + i, width, h - i);
                        }
                        if (fade) {
                            frame.setOpacity(1-i/h);
                        }
                        animating = i < h;
                        if (i == h) {
                            var root = this.getRootPane(owner);
                            glass.getParent().remove(glass);
                            frame.getParent().remove(frame);
                            oldGlass = null;
                            glass = null;
                            frame = null;
                            height = h;
                            if (anchor <> null) {
                                x = dx;
                                y = dy;
                            }
                        }
                    }
                }
            }
        }
    }
}

operation InternalWindow.getRootPane(e:UIElement) {
    var result: <<javax.swing.JRootPane>>;
    if (e instanceof Applet) {
        return ((Applet)e).applet.getRootPane();
    }
    if (e instanceof Widget) {
        var c = (<<java.awt.Component>>)((Widget)e).getComponent();
        while (c <> null) {
            if (c instanceof JApplet) {
                result = ((JApplet)c).getRootPane();
                return result;
            } else if (c instanceof JDialog) {
                result = ((JDialog)c).getRootPane();
                return result;
            } else if (c instanceof JWindow) {
                result = ((JWindow)c).getRootPane();
                return result;
            } else if (c instanceof JFrame) {
                result = ((JFrame)c).getRootPane();
                return result;
            }
            c = c.getParent();
        }
    }
    var w = e.getWindow();
    if (w instanceof JDialog) {
        result = ((JDialog)w).getRootPane();
    } else if (w instanceof JFrame) {
        result = ((JFrame)w).getRootPane();
    } else if (w instanceof JWindow) {
        result = ((JWindow)w).getRootPane();
    } else {
    }
    return result;
}
