/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.canvas.Transformable;
import java.awt.geom.AffineTransform;
import java.awt.Color as AWTColor;
/**
 * A linear gradient consists of a continuously smooth color transition along 
 * a line from one color to another, possibly followed by additional 
 * transitions along the same line to other colors.
 * The values (x1, y1, x2, y2) define the line onto which gradient "stops" 
 * are mapped. The values of x1, y1, x2, y2 are numbers between zero and 1,
 * defining percentage offsets with respect to the target bounds that will
 * be painted by this gradient.
 */

public class LinearGradient extends Gradient {
    /** The x coordinate of the start point of the line. */
    public attribute x1: Number;
    /** The y coordinate of the start point of the line. */
    public attribute y1: Number;
    /** The x coordinate of the end point of the line. */
    public attribute x2: Number;
    /** The y coordinate of the end point of the line. */
    public attribute y2: Number;
    /**
     * Indicates what happens if this gradient starts or ends inside the bounds
     * of the target rectangle. Possible values are: PAD, which says to use 
     * the terminal colors of the gradient to fill the remainder of the target
     * region, REFLECT, which says to reflect the gradient pattern start-to-
     * end, end-to-start, start-to-end, etc. continuously until the target 
     * rectangle is filled, and REPEAT, which says to repeat the gradient 
     * pattern start-to-end, start-to-end, start-to-end, etc. continuously 
     * until the target region is filled. If the attribute is not specified, 
     * the effect is as if a value of "pad" were specified.
     */
    protected operation createPaint(stopCount:Number,
                                    t:AffineTransform,
                                    x1:Number,
                                    y1:Number,
                                    x2:Number,
                                    y2:Number,
                                    holders:StopHolder*,
                                    spreadMethod:SpreadMethod,
                                    gradientUnits:GradientUnits):<<java.awt.Paint>>;
    protected operation createColor(stopCount:Number,
                                    t:AffineTransform,
                                    x1:Number,
                                    y1:Number,
                                    x2:Number,
                                    y2:Number,
                                    holders:StopHolder*,
                                    spreadMethod:SpreadMethod, 
                                    gradientUnits:GradientUnits):AWTColor;
}

attribute LinearGradient.spreadMethod = PAD;
attribute LinearGradient.x2 = 1.0;
attribute LinearGradient.gradientUnits = OBJECT_BOUNDING_BOX;

operation LinearGradient.createPaint(stopCount: Number, 
                                     t:AffineTransform,
                                     x1:Number,
                                     y1:Number,
                                     x2:Number,
                                     y2:Number,
                                     holders:StopHolder*,
                                     spreadMethod:SpreadMethod,
                                     gradientUnits:GradientUnits) {
    var g = new <<net.java.javafx.ui.LinearGradient>>();
    if (affineTransform <> null) {
        g.setTransform(affineTransform);
    }
    g.setGradientUnits(gradientUnits.id);
    g.x1 = x1;
    g.y1 = y1;
    g.x2 = x2;
    g.y2 = y2;
    g.spreadMethod = spreadMethod.id;
    g.stops = foreach (s in holders) s.jstop;
    return g.getPaint();
}

operation LinearGradient.createColor(stopCount:Number,
                                     t:AffineTransform,
                                     x1:Number,
                                     y1:Number,
                                     x2:Number,
                                     y2:Number,
                                     holders:StopHolder*,
                                     spreadMethod:SpreadMethod,
                                     gradientUnits:GradientUnits) {
    var g = new <<net.java.javafx.ui.LinearGradient>>();
    if (affineTransform <> null) {
        g.setTransform(affineTransform);
    }
    g.setGradientUnits(gradientUnits.id);
    g.x1 = x1;
    g.y1 = y1;
    g.x2 = x2;
    g.y2 = y2;
    g.spreadMethod = spreadMethod.id;
    g.stops = foreach (s in holders) s.jstop;
    return g;
}

function LinearGradient.getPaint() {
    return createPaint(stopCount, affineTransform, x1, y1, x2, y2, holders, spreadMethod, gradientUnits);
}

function LinearGradient.getColor() {
    return createColor(stopCount, affineTransform, x1, y1, x2, y2, holders, spreadMethod, gradientUnits);
}

