/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;

import java.awt.Dimension;
import javafx.ui.ScrollableWidget;
import javafx.ui.TableAutoResizeMode;
import javafx.ui.TableCell;
import javafx.ui.TableColumn;


public class Table extends ScrollableWidget {
    private attribute inSetSelection:Boolean;
    private attribute selectionListener:<<javax.swing.event.ListSelectionListener>>;
    private attribute selectionGeneration: Number;
    private attribute tableModel: <<net.java.javafx.ui.UIContextImpl$XTableCellModel>>;
    private attribute table:<<javax.swing.JTable>>;
    private attribute rowcount: Number;
    private attribute dirty: Boolean;

    public attribute locked: Boolean;
    public attribute autoResizeMode: TableAutoResizeMode;

    public attribute visibleRowCount: Number;

    public attribute selection: Number;

    public attribute rowHeight: Number?;
    public attribute rowMargin: Number?;
    public attribute rowSelectionAllowed:Boolean?;

    public attribute showGrid: Boolean;
    public attribute showHorizontalLines: Boolean;
    attribute showVerticalLines: Boolean;
    public attribute onSelectionChange: function();

    public attribute gridColor: Color?;
    public attribute columns: TableColumn+;
    public attribute cells: TableCell*;
    public attribute intercellSpacing: Dimension?;
    protected operation createTable(): <<javax.swing.JTable>>;
}

attribute Table.selection = -1;

trigger on (Table.autoResizeMode = value) {
    if (value <> null) {
        table.setAutoResizeMode(value.id);
    }
}

trigger on (Table.selection[old] = value) {
    if (table <> null) {
        var model = table.selectionModel;
        if (old >= 0 and old < table.getRowCount()) {
            var c1 = old * sizeof columns;
            var c2 = c1 + sizeof columns;
            for (i in [c1..c2-1]) {
                var c = cells[i];
                c.table = null;
                c.selected = false;
                c.table = this;
            }
        }
        if (value < 0 or value >= table.getRowCount()) {
            model.clearSelection();
            selection = -1;
        } else {
            if (table.getSelectedRow() <> value) {
                inSetSelection = true;
                table.setRowSelectionInterval(value, value);
                inSetSelection = false;
            }
            if (true) {
                var c1 = value * sizeof columns;
                var c2 = c1 + sizeof columns;
                for (i in [c1..c2-1]) {
                    var c = cells[i];
                    c.table = null;
                    c.selected = true;
                    c.table = this;
                }
            }
        }
        //onSelectionChange(selection);
    }
}


trigger on (Table.showGrid = value) {
    table.setShowGrid(value);
}

trigger on (Table.rowHeight = value) {
    table.setRowHeight(value);
}

trigger on (Table.rowMargin = value) {
    table.setRowMargin(value);
}

trigger on (Table.rowSelectionAllowed = value) {
    table.setRowSelectionAllowed(value);
}

trigger on (Table.showVerticalLines = value) {
    table.setShowVerticalLines(value);
}

trigger on (Table.showHorizontalLines = value) {
    table.setShowHorizontalLines(value);
}

trigger on (Table.gridColor = value) {
    table.setGridColor(value.getColor());
}

operation Table.createView():<<javax.swing.JComponent>> {
    tableModel = new <<net.java.javafx.ui.UIContextImpl$XTableCellModel>>();
    tableModel.setColumnNames(columns.text);        
    tableModel.setColumnAlignments(columns.alignment.id); 
    for (i in cells) {
        tableModel.addCell(indexof i, i.text, i.toolTipText,
                           i.font.getFont(), i.background.getColor(), i.foreground.getColor(),
                           i.border.getBorder());
    }
    table.setModel(tableModel);
    var columnModel = table.getColumnModel();
    for (i in [0..<table.columnCount]) {
        var col = columns[i];
        if (col.width <> null) {
            columnModel.getColumn(i).setPreferredWidth(col.width);
        }
    }
    if (autoResizeMode <> null) {
        //table.setAutoResizeMode(autoResizeMode.id);
    }
    if (intercellSpacing <> null) {
        table.setIntercellSpacing(intercellSpacing);
    }
    if (selection >= 0 and selection < table.getRowCount()) {
        table.clearSelection();
        table.setRowSelectionInterval(selection, selection);
    }
    return table;
}

operation Table.createTable() {
    return context:<<net.java.javafx.ui.UIContext>>.createTable();
}

trigger on (new Table) {
    var self = this;
    showHorizontalLines = true;
    showVerticalLines = true;
    showGrid = true;
    table = this.createTable();
    context:<<net.java.javafx.ui.UIContext>>.installXTableCellRenderer(table);
    table.setOpaque(false);
    //table.getTableHeader().setOpaque(true);
    table.setSelectionMode(table.selectionModel.SINGLE_SELECTION);
    context:<<net.java.javafx.ui.UIContext>>.installTableHeaderRenderer(table);

    selectionListener = new <<javax.swing.event.ListSelectionListener>>() {
                                  operation valueChanged(e:<<javax.swing.event.ListSelectionEvent>>) {
                                      if (self.inSetSelection) {
                                          return;
                                      }
                                      if (not e.valueIsAdjusting) {
                                          var n = ++self.selectionGeneration;
                                          var row = self.table.getSelectedRow();
                                          do later {
                                              if (self.selectionGeneration == n and self.table.getSelectedRow() == row) {
                                                  self.selection = row;
                                              }
                                          }
                                      }
                                  }
                              };
    table.getSelectionModel().addListSelectionListener(selectionListener);
}


trigger on (insert cell into Table.cells) {
    cell.table = this;
    if (tableModel <> null) {
        var s = selection;
        tableModel.addCell(indexof cell, cell.text, cell.toolTipText,
                           cell.font.getFont(), cell.background.getColor(), cell.foreground.getColor(),
                           cell.border.getBorder());
        selection = s;
    }
}


trigger on (delete cell from Table.cells) {
    if (tableModel <> null) {
        var s = selection;
        tableModel.removeCell(indexof cell);
        selection = s;
    }
}

trigger on (Table.intercellSpacing = n) {
    table.setIntercellSpacing(n);
}
