/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import net.java.javafx.ui.UIContext;
import java.lang.System;


// Default Animation Functions

operation __EASEBOTH(t:Number) {
    var timer = new <<net.java.javafx.typeImpl.TimerImpl>>(0, 1000, 1000);
    timer.setEaseBoth();
    var timeDiff = t * 1000;
    return timer.calcNextValue(timeDiff, 0)/1000;
}

operation __EASEIN(t:Number) {
    var timer = new <<net.java.javafx.typeImpl.TimerImpl>>(0, 1000, 1000);
    timer.setEaseIn();
    var timeDiff = t * 1000;
    return timer.calcNextValue(timeDiff, 0)/1000;
}

operation __EASEOUT(t:Number) {
    var timer = new <<net.java.javafx.typeImpl.TimerImpl>>(0, 1000, 1000);
    timer.setEaseOut();
    var timeDiff = t * 1000;
    return timer.calcNextValue(timeDiff, 0)/1000;
}

operation __EASE(a:**, t:Number, f(Number):Number, interpolate(value1, value2, t:Number)) {
    t = f(t);
    var off = t * (sizeof a-1);
    var i = off.intValue();
    var value1 = a[i];
    if (i + 1 == sizeof a) {
        return value1;
    }
    var value2 = a[i+1];
    return interpolate(value1, value2, off-i);
}

operation __INTERPOLATE_NUM(value1:Number, value2: Number, t:Number) {
    return value1 + (value2-value1)*t;
}

EASEBOTH:Number = operation(a:Number*, t:Number) {
    return __EASE(a, t, __EASEBOTH, __INTERPOLATE_NUM);
};

EASEIN:Number = operation(a:Number*, t:Number) {
    return __EASE(a, t, __EASEIN, __INTERPOLATE_NUM);
};

EASEOUT:Number = operation(a:Number*, t:Number) {
    return __EASE(a, t, __EASEOUT, __INTERPOLATE_NUM);
};

LINEAR:Number = operation(a:Number*, t:Number) {
    return __EASE(a, t, function(t:Number) {return t;}, __INTERPOLATE_NUM);
};

DISCRETE:Number = operation(a:Number*, t:Number) {
    return a[t*(sizeof a -1)];
};


EASEBOTH:Integer = operation(a:Integer*, t:Number) {
    return __EASE(a, t, __EASEBOTH, __INTERPOLATE_NUM);
};

EASEIN:Integer = operation(a:Integer*, t:Number) {
    return __EASE(a, t, __EASEIN, __INTERPOLATE_NUM);
};

EASEOUT:Integer = operation(a:Integer*, t:Number) {
    return __EASE(a, t, __EASEOUT, __INTERPOLATE_NUM);
};

LINEAR:Integer = operation(a:Integer*, t:Number) {
    return __EASE(a, t, function(t:Number) {return t;}, __INTERPOLATE_NUM);
};

DISCRETE:Integer = operation(a:Integer*, t:Number) {
    return a[t*(sizeof a -1)];
};




// net.java.javafx.ui.UIContext is a helper class providing shared cell renderers,
// image caching, and a few other hacks to work around current Swing
// limitations. The same package also contains a few non-Swing components:
// Splitter layout, Stack layout, InfiniteProgressPanel, ShadowedBorder

try {
    //<<javax.swing.JPopupMenu>>.setDefaultLightWeightPopupEnabled(false);
    <<javax.swing.UIManager>>.put("swing.boldMetal", false);
    var osName = System.getProperty("os.name").toLowerCase();
    if (osName.contains("win") or osName.contains("mac")) {
       <<javax.swing.UIManager>>.setLookAndFeel(<<javax.swing.UIManager>>.getSystemLookAndFeelClassName());
    }
} catch (e) {
}

context:<<net.java.javafx.ui.UIContext>> = new <<net.java.javafx.ui.UIContextImpl>>();
context:<<net.java.javafx.ui.UIContext>>.getModule().disableUndo();
context:<<net.java.javafx.ui.UIContext>>.getModule().disableExtent();
//context:<<net.java.javafx.ui.UIContext>>.getModule().enableExtent();
//context:<<net.java.javafx.ui.UIContext>>.debugFocus();

public class UIElement {
    public operation getWindow():<<java.awt.Window>>;
    public attribute lookAndFeel: String?;
}

trigger on (UIElement.lookAndFeel = value) {
    <<javax.swing.UIManager>>.setLookAndFeel(value);
    <<javax.swing.SwingUtilities>>.updateComponentTreeUI(this.getWindow());
}


cmToPixel:Number = function(n:Number) = context:UIContext.centimeterToPixel(n);
mmToPixel:Number = function(n:Number) = context:UIContext.millimeterToPixel(n);
inchToPixel:Number = function(n:Number) = context:UIContext.inchToPixel(n);
pointToPixel:Number = function(n:Number) = context:UIContext.pointToPixel(n);


