/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.UIElement;
import javafx.ui.AbstractFrame;
import javafx.ui.MenuBar;
import javafx.ui.Dialog;
import javafx.ui.Image;
import net.java.javafx.ui.UIContext;

class Window extends AbstractFrame {
    public attribute disposeOnClose: Boolean;
    attribute screenx: Number;
    attribute screeny: Number;
    attribute owner: UIElement?;
    attribute frame: <<javax.swing.JWindow>>;
    attribute menubar: MenuBar?;
    attribute content: Widget;
    attribute title: String;
    attribute height: Number?;
    attribute width: Number?;
    operation show();
    operation hide();
    operation pack();
    operation showDialog(d:Dialog);
    attribute onClose: function();
    attribute visible: Boolean;
    attribute resizable: Boolean;
    attribute iconImage: Image?;
    attribute undecorated: Boolean;
    operation setContentPane(w:Widget);
}

attribute Window.resizable = true;
attribute Frame.disposeOnClose = true;

trigger on Window.screenx = n {
    frame.setLocation(new <<java.awt.Point>>(screenx, screeny));
}

trigger on Window.screeny = n {
    frame.setLocation(new <<java.awt.Point>>(screenx, screeny));
}

trigger on (Window.iconImage = value) {

}

trigger on (Window.resizable = value) {

}

trigger on (Window.undecorated = value) {

}

trigger on (Window.visible = value) {
     if (value) {
        this.pack();
        do later {
            this.show();
        }
     } else {
        frame.visible = false;
     }
}

trigger on (Window.menubar = value) {

}

trigger on (Window.title = value) {

}

trigger on (Window.content = value) {
    this.setContentPane(value);
}

operation Window.setContentPane(w:Widget) {
    frame.setContentPane(w.getComponent());
    frame.validate();
}

operation Window.pack() {
    frame.pack();
}


trigger on (Window.height = value) {
    if (value <> null) {
        var dim = frame.getSize();
        dim.height = value;
        frame.setSize(dim);
    }
}

trigger on (Window.width = value) {
    if (value <> null) {
        var dim = frame.getSize();
        dim.width = value;
        frame.setSize(dim);
    }
}

operation Window.show() {
    this.pack();
    var dim = frame.getSize();
    if (height <> null) {
        dim.height = height;
    }
    if (width <> null) {
        dim.width = width;
    }
    frame.setSize(dim);
    frame.setLocation(-2000, -2000);
    frame.visible = true;
    frame.visible = false;
    if (owner <> null) {
        frame.setLocationRelativeTo(owner.getWindow());
    } else {
        frame.setLocation(screenx, screeny);
    }
    frame.visible = true;
}

operation Window.hide() {
    frame.visible = false;
}

operation Window.close() {
    frame.dispose();
    frame = null;
}

trigger on (new Window) {
    frame = new <<javax.swing.JWindow>>;
    context:UIContext.registerWindow(frame);
    frame.setBackground(new <<java.awt.Color>>(0,0,0,0));
    win = frame;
    var self = this;
    win.addWindowListener(new <<java.awt.event.WindowListener>>() {
                              operation windowClosing(e) {
                                  (self.onClose)();
                                  context:UIContext.unregisterWindow(self.frame);
                                  if (self.disposeOnClose) {
                                      self.frame.dispose();
                                  }
                              }
                          });
}
