/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import net.java.javafx.jazz.event.ZNodeListener;
import net.java.javafx.jazz.event.ZNodeEvent;
import net.java.javafx.jazz.ZLayoutManager;
import net.java.javafx.jazz.ZLayoutGroup;
import net.java.javafx.jazz.ZGroup;
import java.lang.Math;

class FlowLayout extends Layout {
    attribute locked: Boolean;
    attribute layoutRequests: Number;
    attribute inLayout: Boolean;
    attribute nodelistener: ZNodeListener;
    attribute layoutManager: ZLayoutManager;
    attribute gap: Number;
    attribute vgap: Number;
    attribute width: Number?;
    attribute animated: Boolean;
    attribute animationDuration: Number;
}

attribute FlowLayout.animationDuration = 1000;
attribute FlowLayout.gap = 5;
attribute FlowLayout.vgap = 5;

trigger on FlowLayout.locked = value {
    if (not value) {
        this.doLayout();
    } 
}

operation FlowLayout.doLayout() {
    if (locked) {
        return;
    }
    if (inLayout) {
        return;
    }     
    inLayout = true;
    var x = 0.0;
    var maxh = 0.0;
    var y = 0;
    var paths: Line* = [];
    for (i in content) {
        var b = i.getNode().getBounds();
        maxh = Math.max(maxh, b.getHeight());
        if (width <> null and x + b.getWidth() > width) {
            y += maxh + vgap;
            maxh = 0.0;
            x = 0;
        }
        var t = i.tg.editor().getTransformGroup();
        insert Line {
             x1: t.getTranslateX()
             y1: t.getTranslateY()
             x2: x
             y2: y
        } into paths;
        if (not animated) {
            t.setTranslateX(x);
            t.setTranslateY(y);
        }
        x += b.getWidth() + gap;
    }
    if (animated) {
        var req = ++layoutRequests;
        for (t in [1..100]) (dur animationDuration while req == layoutRequests) {
            var r  = t/100;
            for (i in content) {
                var tg = i.tg.editor().getTransformGroup();
                var line = paths[indexof i];
                var x = line.x1 + (line.x2 - line.x1)*r;
                var y = line.y1 + (line.y2 - line.y1)*r;
                tg.setTranslation(x, y);
            }
            inLayout = t < 100;
        }
    } else {
        inLayout = false;
    }
}

trigger on FlowLayout.width = value {
    layoutgroup.invalidate();
}

operation FlowLayout.createNode() {
    layoutgroup = new ZLayoutGroup();
    var self = this;
    nodelistener = new ZNodeListener() {
            operation boundsChanged(e:ZNodeEvent) {
                if (self.locked) {
                   e.consume();
                   return;
                }
                if (not self.inLayout) {
                   self.layoutgroup.invalidate();
                }
                e.consume();
            }
        };
    layoutManager = new ZLayoutManager() {
            operation doLayout(g:ZGroup) {
                self.doLayout();
            }
            operation preLayout(g:ZGroup) {
            }
            operation postLayout(g:ZGroup) {
            }
        };
    layoutgroup.setLayoutManager(layoutManager);
    layoutgroup.addNodeListener(nodelistener);

    for (i in content) {
        var n = i.getNode();
        //n.addNodeListener(nodelistener);
        layoutgroup.addChild(n);
    }
    return layoutgroup;
}

