/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.PathElement;
import java.awt.geom.GeneralPath;

/**
 * Adds a point to the path by moving to the specified
 * coordinates (x, y).
 */

public class MoveTo extends PathElement {
    /** the x coordinate of the point */
    public attribute x: Number;
    /** the y coordinate of the point */
    public attribute y: Number;        
}

trigger on MoveTo.x = value {
    path.buildPath();
}

trigger on MoveTo.y = value {
    path.buildPath();
}

attribute MoveTo.absolute = true;

operation MoveTo.addTo(gp:GeneralPath) {
    if (absolute) {
    	path.currentX = x;
    	path.xCenter = path.currentX;
    	
    	path.currentY = y;
    	path.yCenter = path.currentY;
    
        gp.moveTo(x, y);
        path.lastMoveToX = x;
        path.lastMoveToY = y;
    } else {
    	path.currentX += x;
    	path.xCenter = path.currentX;
    	
    	path.currentY += y;
    	path.yCenter = path.currentY;
    
        var pt = gp.getCurrentPoint();
        gp.moveTo(pt.x + x, pt.y + y);
        path.lastMoveToX = pt.x + x;
        path.lastMoveToY = pt.y + y;
    }
}


