/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.AbstractPathElement;
import java.awt.geom.GeneralPath;
import net.java.javafx.jazz.component.ZPath;

/**
 * The <code>Path</code> class represents a geometric path 
 * constructed from straight lines, and quadratic and cubic
 * (B&eacute;zier) curves.
 */

class Path extends Shape {
    attribute locked: Boolean;
    attribute lastMoveToX: Number?;
    attribute lastMoveToY: Number?;
    attribute xCenter: Number?;
    attribute yCenter: Number?;
    attribute currentX: Number?;
    attribute currentY: Number?;
    private attribute zpath: ZPath;
    protected operation buildPath();
    /**
     * Path Data<br></br>
     * A list of commands and/or shapes which define this path.
     */
    public attribute d: AbstractPathElement*;
}

attribute Path.fillRule = NON_ZERO:FillRule;

trigger on Path.locked = value {
    if (not value) {
        buildPath();
    }
}

operation Path.buildPath() {
    if (zpath <> null and not locked) {
        var gp = new GeneralPath(fillRule.id, sizeof d);
        for (i in d) {
            i.addTo(gp);
        }
        zpath.setPath(gp);
    }
}

operation Path.toPath() {
    return this;
}

operation Path.createShape() {
    zpath = new ZPath();
    this.buildPath();
    return zpath;
}

trigger on insert p into Path.d {
    p.path = this;
    this.buildPath();
}

trigger on delete p from Path.d {
    this.buildPath();
}

trigger on Path.d[oldValue] = newValue {
    newValue.path = this;
    this.buildPath();
}
