/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import java.awt.TexturePaint;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.geom.Rectangle2D$Double as Rectangle2D;
import java.awt.Color;
import javafx.ui.Canvas;
import javafx.ui.Paint;
/**
 * A Pattern is used to fill or stroke an object using a pre-defined graphic 
 * object which can be replicated ("tiled") at fixed intervals in x and y to 
 * cover the areas to be painted. 
  <p>
 * Attributes x, y, width, height define a reference rectangle somewhere on 
 * the infinite canvas. The reference rectangle has its top/left at (x,y) and 
 * its bottom/right at (x+width,y+height). The tiling theoretically extends a 
 * series of such rectangles to infinity in X and Y (positive and negative), 
 * with rectangles starting at (x + m*width, y + n*height) for each possible 
 * integer value for m and n.
*/

public class Pattern extends Paint {
    private attribute canvas: Canvas?;
    private attribute texturePaint: TexturePaint;
    /** The x coordinate of the start point of the tile. Defaults to zero. */
    public attribute x: Number;
    /** The y coordinate of the start point of the tile. Defaults to zero */
    public attribute y: Number;
    /** The width the tile. Defaults to the preferred size of this pattern's content. */
    public attribute width: Number?;
    /** The height of the tile. Defaults to the preferred size of this pattern's content. */
    public attribute height: Number?;
    /** The graphic objects that will be used to create the tile that this pattern will paint.*/
    public attribute content: Node*;
}

operation Pattern.makeTexture() {
    if (canvas == null) {
        canvas = Canvas {content: bind content};
    }
    if (texturePaint == null) {
        canvas.getComponent();
        var im ;
        try {
            im  = (BufferedImage)canvas.zcanvas.getIconImage();
        } catch (e) {
            return;            
        }
        var w = im.getWidth();
        var h = im.getHeight();
        if (height <> null) {
            h = height;
        }
        if (width <> null) {
            w = width;
        }
        if (im <> null) {
            var rect = new Rectangle2D(x, y, w, h);
            texturePaint = new TexturePaint(im, rect);
        }
    }
}

operation Pattern.getPaint() {
    makeTexture();
    if (texturePaint == null) {
        return black:Color;
    }
    return texturePaint;
}

