/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.PathElement;
import java.awt.geom.GeneralPath;

/**
 * Adds a curved segment, defined by two new points, to the path by
 * drawing a Quadratic curve that intersects both the current
 * coordinates and the coordinates (x2,&nbsp;y2), using the 
 * specified point (x1,&nbsp;y1) as a quadratic parametric control
 * point.
 */

class QuadTo extends PathElement {
    public attribute smooth: Boolean;

    /** the x coordinate of the first quadratic control point */
    attribute x1: Number;
    /** the y coordinate of the first quadratic control point */
    attribute y1: Number;
    /** the x coordinate of the second quadratic control point */
    attribute x2: Number;
    /** the y coordinate of the second quadratic control point */
    attribute y2: Number;
}

operation QuadTo.addTo(gp:GeneralPath) {
	if (smooth) {
        if (absolute) {
            path.xCenter = path.currentX * 2 - path.xCenter;
            path.yCenter = path.currentY * 2 - path.yCenter;
 
            path.currentX = x2;
            path.currentY = y2;
                 		
            gp.quadTo(path.xCenter, path.yCenter, path.currentX, path.currentY);
        } else {
            path.xCenter = path.currentX * 2 - path.xCenter;
            path.yCenter = path.currentY * 2 - path.yCenter;

            path.currentX += x2;
            path.currentY += y2;
                 		
			gp.quadTo(path.xCenter, path.yCenter, path.currentX, path.currentY);
        }

		return;
	}
	
	if (absolute)  {
		path.xCenter = x1;
		path.yCenter = y1;
		path.currentX = x2;
		path.currentY = y2;
	} else {
		path.xCenter = path.currentX + x1;
		path.yCenter = path.currentY + y1;
		path.currentX += x2;
		path.currentY += y2;
	}
	
    gp.quadTo(x1, y1, x2, y2);
}

trigger on QuadTo.x1 = value {
    path.buildPath();
}

trigger on QuadTo.x2 = value {
    path.buildPath();
}

trigger on QuadTo.y1 = value {
    path.buildPath();
}

trigger on QuadTo.y2 = value {
    path.buildPath();
}
